<?php
namespace Auction\Entity;

use Doctrine\ORM\Mapping as ORM;


/**
 * An Auction item.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="auction")
 */
class Auction
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="created_by_profile_id")
	 **/
	protected $createdBy;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=false, name="company_id")
	 **/
	protected $company;

	/**
	 * @ORM\ManyToOne(targetEntity="\Stock\Entity\Stock")
	 * @ORM\JoinColumn(nullable=false, name="stock_id")
	 **/
	protected $stock;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=true, name="sold_to_profile_id")
	 **/
	protected $soldToProfile;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=true, name="sold_to_company_id")
	 **/
	protected $soldToCompany;


	/* ------------------------------------ Winning bid ------------------------------------ */
	/**
	 * @ORM\OneToOne(targetEntity="\Auction\Entity\Bid", cascade={"all"})
	 * @ORM\JoinColumn(nullable=true, name="auction_bid_id")
	 **/
	protected $currentBid;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="bid_price", options={"unsigned"=true});
	 */
	protected $currentBidPrice = 0.0;


	/* ------------------------------------ Data ------------------------------------ */
	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="reserve_price",
	 *                             options={"unsigned"=true});
	 */
	protected $reservePrice = 0.0;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="bid_increment",
	 *                             options={"unsigned"=true});
	 */
	protected $bidIncrement = 0.0;

	/**
	 * @ORM\Column(type="datetime", name="start_date");
	 */
	protected $startDate;

	/**
	 * @ORM\Column(type="datetime", name="end_date");
	 */
	protected $endDate;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="number_of_bids");
	 */
	protected $numberOfBids = 0;


	/* ------------------------------------ Linkage ------------------------------------ */
	/**
	 * @ORM\OneToMany(targetEntity="Basket", mappedBy="auction", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $baskets;

	/**
	 * @ORM\OneToMany(targetEntity="Bid", mappedBy="auction", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $bids;


	/* ------------------------------------ De-normalization ------------------------------------ */
	/* ------------------------------------ Searchable */

	/**
	 * @ORM\Column(type="integer", name="region_id", nullable=true);
	 */
	protected $regionId;

	/**
	 * @ORM\Column(type="integer", name="make_id", nullable=true);
	 */
	protected $makeId;

	/**
	 * @ORM\Column(type="integer", name="auction_company_id", nullable=true);
	 */
	protected $auctionCompanyId;

	/**
	 * @ORM\Column(type="string", name="auction_company_name", length=100, nullable=true)
	 */
	protected $auctionCompanyName;

	/**
	 * @ORM\Column(type="string", name="model_name", length=75, nullable=true)
	 */
	protected $modelName;

	/**
	 * @ORM\Column(type="string", name="type_name", length=75, nullable=true)
	 */
	protected $typeName;

	/**
	 * @ORM\Column(type="string", length=12, nullable=true, name="reference_no", nullable=true)
	 */
	protected $referenceNumber;

	/**
	 * @ORM\Column(type="string", length=15, name="registration_number", nullable=true)
	 */
	protected $registrationNumber;

	/**
	 * @ORM\Column(type="integer", nullable=true, options={"unsigned"=true}, nullable=true)
	 */
	protected $stockKm;

	/* ------------------------------------ Not Searchable */

	/**
	 * @ORM\Column(type="integer", name="current_bid_company_id", nullable=true);
	 */
	protected $currentBidCompanyId;

	/**
	 * @ORM\Column(type="integer", name="stock_company_id", nullable=true);
	 */
	protected $stockCompanyId;

	/**
	 * @ORM\Column(type="string", name="region_name", length=75, nullable=true)
	 */
	protected $regionName;

	/**
	 * @ORM\Column(type="string", name="make_name", length=75, nullable=true)
	 */
	protected $makeName;

	/**
	 * @ORM\Column(type="string", name="year_name", length=4, nullable=true)
	 */
	protected $yearName;

	/**
	 * @ORM\Column(type="datetime", name="stock_created", nullable=true);
	 */
	protected $stockCreated;



	/* ------------------------------------ Tracking ------------------------------------ */
	/**
	 * @ORM\Column(type="boolean", name="email_sent");
	 */
	protected $emailSent = false;

	/**
	 * @ORM\Column(nullable=true, type="string", length=25, name="status");
	 */
	protected $jobState = 'Active';

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->baskets = new \Doctrine\Common\Collections\ArrayCollection();
		$this->bids    = new \Doctrine\Common\Collections\ArrayCollection();
	}

	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed  $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
		if (is_null($this->startDate))
		{
			$this->startDate = new \DateTime("now");
		}
		$this->regionId            = $this->company->city->region->id;
		$this->regionName          = $this->company->city->region->name;
		$this->auctionCompanyId    = $this->company->id;
		$this->auctionCompanyName  = $this->company->name;
		$this->makeId              = $this->stock->type->model->make->id;
		$this->makeName            = $this->stock->type->model->make->name;
		$this->modelName           = $this->stock->type->model->name;
		$this->typeName            = $this->stock->type->name;
		$this->yearName            = $this->stock->vehicleYear->name;
		$this->referenceNumber     = $this->stock->referenceNumber;
		$this->registrationNumber  = $this->stock->registrationNumber;
		$this->stockKm             = $this->stock->km;
		$this->stockCompanyId      = $this->stock->company->id;
		$this->stockCreated        = $this->stock->created;
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
		if (!is_null($this->currentBid))
		{
			$this->currentBidCompanyId = $this->currentBid->company->id;
		}
	}

	/**
	 * Convert the object to an array.
	 * @param array   $expand
	 * @param array   $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
		array $expand = array(), array $intersect = array(),
		$showIdentifiers = false, $expandAll = 0
	)
	{
		$intersect      = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll     = empty($intersect);
		$data           = array();
		($includeAll || isset($intersect['id']))
		&& $data['id'] = $this->id;
		($includeAll || isset($intersect['createdBy']))
		&& $data['createdBy'] = (in_array('createdBy', $expand) || $expandAll || $showIdentifiers)
		                        && !is_null($this->createdBy)
			? (!$showIdentifiers ? $this->createdBy->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->createdBy->id)
			: null;
		($includeAll || isset($intersect['company']))
		&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
		                      && !is_null($this->company)
			? (!$showIdentifiers ? $this->company->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->company->id)
			: null;
		($includeAll || isset($intersect['stock']))
		&& $data['stock'] = (in_array('stock', $expand) || $expandAll || $showIdentifiers)
		                    && !is_null($this->stock)
			? (!$showIdentifiers ? $this->stock->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->stock->id)
			: null;
		($includeAll || isset($intersect['soldToProfile']))
		&& $data['soldToProfile'] = (in_array('soldToProfile', $expand) || $expandAll || $showIdentifiers)
		                            && !is_null($this->soldToProfile)
			? (!$showIdentifiers ? $this->soldToProfile->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->soldToProfile->id)
			: null;
		($includeAll || isset($intersect['soldToCompany']))
		&& $data['soldToCompany'] = (in_array('soldToCompany', $expand) || $expandAll || $showIdentifiers)
		                            && !is_null($this->soldToCompany)
			? (!$showIdentifiers ? $this->soldToCompany->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->soldToCompany->id)
			: null;
		($includeAll || isset($intersect['currentBid']))
		&& $data['currentBid'] = (in_array('currentBid', $expand) || $expandAll || $showIdentifiers)
		                         && !is_null($this->currentBid)
			? (!$showIdentifiers ? $this->currentBid->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
			) : $this->currentBid->id)
			: null;
		($includeAll || isset($intersect['currentBidPrice']))
		&& $data['currentBidPrice'] = $this->currentBidPrice;
		($includeAll || isset($intersect['reservePrice']))
		&& $data['reservePrice'] = $this->reservePrice;
		($includeAll || isset($intersect['bidIncrement']))
		&& $data['bidIncrement'] = $this->bidIncrement;
		($includeAll || isset($intersect['startDate']))
		&& $data['startDate'] = !is_null($this->startDate)
			? $this->startDate->format($dateTimeFormat)
			: null;
		($includeAll || isset($intersect['endDate']))
		&& $data['endDate'] = !is_null($this->endDate)
			? $this->endDate->format($dateTimeFormat)
			: null;
		($includeAll || isset($intersect['numberOfBids']))
		&& $data['numberOfBids'] = $this->numberOfBids;
		($includeAll || isset($intersect['baskets']))
		&& $data['baskets'] = (in_array('baskets', $expand) || $expandAll || $showIdentifiers)
		                      && !is_null($this->baskets)
			? $this->basketsToArray($showIdentifiers)
			: null;
		($includeAll || isset($intersect['bids']))
		&& $data['bids'] = (in_array('bids', $expand) || $expandAll || $showIdentifiers)
		                   && !is_null($this->bids)
			? $this->bidsToArray($showIdentifiers)
			: null;
		($includeAll || isset($intersect['jobState']))
		&& $data['jobState'] = $this->jobState;
		($includeAll || isset($intersect['created']))
		&& $data['created'] = !is_null($this->created)
			? $this->created->format($dateTimeFormat)
			: null;
		($includeAll || isset($intersect['updated']))
		&& $data['updated'] = !is_null($this->updated)
			? $this->updated->format($dateTimeFormat)
			: null;
		return $data;
	}

	/**
	 * Internal utility to change auction baskets collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	protected function basketsToArray($showIdentifiers = false)
	{
		$data     = array();
		$iterator = $this->baskets->getIterator();
		foreach ($iterator as $basket)
		{
			$data[] = $showIdentifiers
				? array('id' => $basket->id)
				: $basket->toArray();
		}
		return $data;
	}

	/**
	 * Internal utility to change auction bids collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	protected function bidsToArray($showIdentifiers = false)
	{
		$data     = array();
		$iterator = $this->bids->getIterator();
		foreach ($iterator as $bid)
		{
			$data[] = $showIdentifiers
				? array('id' => $bid->id)
				: $bid->toArray();
		}
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
		&& $this->id = $data['id'];
		isset($data['createdBy'])
		&& $this->createdBy = $data['createdBy'];
		isset($data['company'])
		&& $this->company = $data['company'];
		isset($data['stock'])
		&& $this->stock = $data['stock'];
		isset($data['soldToProfile'])
		&& $this->soldToProfile = $data['soldToProfile'];
		isset($data['soldToCompany'])
		&& $this->soldToCompany = $data['soldToCompany'];
		isset($data['currentBid'])
		&& $this->currentBid = $data['currentBid'];
		isset($data['currentBidPrice'])
		&& $this->currentBidPrice = $data['currentBidPrice'];
		isset($data['reservePrice'])
		&& $this->reservePrice = $data['reservePrice'];
		isset($data['bidIncrement'])
		&& $this->bidIncrement = $data['bidIncrement'];
		isset($data['startDate'])
		&& $this->startDate = $data['startDate'];
		isset($data['endDate'])
		&& $this->endDate = $data['endDate'];
		isset($data['numberOfBids'])
		&& $this->numberOfBids = $data['numberOfBids'];
	}



}

