<?php
namespace Workspace\Service;



/**
 * Basic data node to provide global functionality to workflow services.
 * @author andre.fourie
 */
class Workflow
{
	/**
	 * Contract types
	 */
	const CONTRACT_TYPE_RECCURING	= 'Recurring';
	const CONTRACT_TYPE_USE_ONCE	= 'UseOnce';
	
	/**
	 * Field types
	 */
	const FIELD_TYPE_DEFAULT			= 'Default';
	const FIELD_TYPE_ARRAY				= 'Array';
	const FIELD_TYPE_DATE				= 'Date';
	const FIELD_TYPE_EMAIL				= 'Email';
	const FIELD_TYPE_MOBILE				= 'Mobile';
	const FIELD_TYPE_ID					= 'Id';
	const FIELD_TYPE_BOOLEAN			= 'Boolean';
	const FIELD_TYPE_DEPARTMENT			= 'Department';
	const FIELD_TYPE_DEALER_TYPE		= 'DealerType';
	const FIELD_TYPE_DEALER_CLIENT_TYPE	= 'DealerClientType';
	const FIELD_TYPE_COMPANY_TYPE		= 'CompanyType';
	const FIELD_TYPE_POSITION			= 'Position';
	const FIELD_TYPE_USERTYPE			= 'UserType';
	const FIELD_TYPE_ID_NUMBER_ZA		= 'ZaId';
	const FIELD_TYPE_USERNAME			= 'Username';
	const FIELD_TYPE_PASSWORD			= 'Password';
	const FIELD_TYPE_ID_NUMBER			= 'IdNumber';
	const FIELD_TYPE_NAME				= 'Name';
	const FIELD_TYPE_PERSON_NAME		= 'PersonName';
	const FIELD_TYPE_MONTH_DIGIT		= 'MonthDigit';
	const FIELD_TYPE_COLLECTION			= 'Collection';
	
	const FIELD_TYPE_DECIMAL			= 'Decimal';
	const FIELD_TYPE_INTEGER			= 'Integer';
	const FIELD_TYPE_DIGIT_3			= 'Digit3';
	const FIELD_TYPE_DIGIT_4			= 'Digit4';
	const FIELD_TYPE_DIGIT_10			= 'Digit10';
	
	const FIELD_TYPE_STRING_2			= 'String2';
	const FIELD_TYPE_STRING_10			= 'String10';
	const FIELD_TYPE_STRING_12			= 'String12';
	const FIELD_TYPE_STRING_15			= 'String15';
	const FIELD_TYPE_STRING_20			= 'String20';
	const FIELD_TYPE_STRING_25			= 'String25';
	const FIELD_TYPE_STRING_40			= 'String40';
	const FIELD_TYPE_STRING_50			= 'String50';
	const FIELD_TYPE_STRING_75			= 'String75';
	const FIELD_TYPE_STRING_100			= 'String100';
	const FIELD_TYPE_STRING_250			= 'String250';
	const FIELD_TYPE_STRING_500			= 'String500';
	
	/**
	 * @var \Workspace\Workflow
	 */
	protected $workflowNode;
	/**
	 * @var \Doctrine\ORM\EntityManager
	 */
	protected $em;
	/**
	 * @var array
	 */
	protected $meta;


	/**
	 * Standard workflow.
	 * @param \Workspace\Workflow $workflowNode
	 */
	public function setWorkflow(\Workspace\Workflow $workflowNode)
	{
		$this->workflowNode = $workflowNode;
		$this->em = $this->workflowNode->getEntityManager();
	}


	/**
	 * Utility to construct options object from meta data.
	 * @param array $meta
	 * @return \Workspace\UseCase\Options
	 */
	protected function buildOptions(array $meta){
		$options = new \Workspace\UseCase\Options();
		if (isset($meta['Options']) && !empty($meta['Options']))
		{
			if (isset($meta['Options']['Boolean']) && !empty($meta['Options']['Boolean']))
			{
				foreach ($meta['Options']['Boolean'] as $option => $default)
				{
					$options->optBoolean($option, $default);
				}
			}
			isset($meta['Options']['OneFromList'])
			&& !empty($meta['Options']['OneFromList'])
			&& $options->optOneFromList(
					$meta['Options']['OneFromList']['Name'],
					$meta['Options']['OneFromList']['List'],
					$meta['Options']['OneFromList']['Default']
			);
			isset($meta['Options']['ManyFromList'])
			&& !empty($meta['Options']['ManyFromList'])
			&& $options->optManyFromList(
					$meta['Options']['ManyFromList']['Name'],
					$meta['Options']['ManyFromList']['List'],
					$meta['Options']['ManyFromList']['Default']
			);
		}
		return $options;
	}

	/**
	 * Utility to construct requirements object from meta data.
	 * @param array $meta
	 * @return \Workspace\UseCase\Requirement
	 */
	protected function buildRequirements(array $meta){
		$requirement = new \Workspace\UseCase\Requirement();
		isset($meta['RequiredInput'])
			&& !empty($meta['RequiredInput'])
			&& $requirement->addRequiredInput($meta['RequiredInput']);
		isset($meta['OptionalInput'])
			&& !empty($meta['OptionalInput'])
			&& $requirement->addOptionalInput($meta['OptionalInput']);
		return $requirement;
	}

	/**
	 * Utility to construct conditional case object(s) from meta data and attach to contract.
	 * @param \Workspace\Contract\AbstractBase $contract
	 * @param array $meta
	 * @return \Workspace\Contract\AbstractBase
	 */
	protected function buildConditions(\Workspace\Contract\AbstractBase $contract, array $meta)
	{
		if (isset($meta['Conditions']) && !empty ($meta['Conditions']))
		{
			foreach ($meta['Conditions'] as $name => $conditionMeta)
			{
				$condition = (isset($conditionMeta['Base']) && 'Or' == $conditionMeta['Base'])
					? new \Workspace\UseCase\ConditionOrBase()
					: new \Workspace\UseCase\ConditionAndBase();
				foreach ($conditionMeta['Condition'] as $newCondition)
				{
					isset($newCondition['Type'])
						|| $newCondition['Type'] = 'equal';
					switch ($newCondition['Type'])
					{
						case 'notEqual':
							$condition->notEqual($newCondition['Field'], $newCondition['Value']);
							break;
						case 'smallerThan':
							$condition->smallerThan($newCondition['Field'], $newCondition['Value']);
							break;
						case 'greaterThan':
							$condition->greaterThan($newCondition['Field'], $newCondition['Value']);
							break;
						case 'oneOf':
							$condition->oneOf($newCondition['Field'], $newCondition['Value']);
							break;
						case 'notOneOf':
							$condition->notOneOf($newCondition['Field'], $newCondition['Value']);
							break;
						case 'equal':
						default:
							$condition->equal($newCondition['Field'], $newCondition['Value']);
							break;
					}
				}
				$requirement = new \Workspace\UseCase\Requirement();
				$requirement->addRequiredInput($conditionMeta['RequiredInput']);
				isset($conditionMeta['OptionalInput'])
					&& !empty($conditionMeta['OptionalInput'])
					&& $requirement->addOptionalInput($conditionMeta['OptionalInput']);
				$contract->addConditionalCase($name, $condition, $requirement);
			}
		}
		return $contract;
	}

}